/***********************************************************************/
/**
$Header: /home/eliot/asi/repo/cvsrepo/Repository/drv/hpi/hpidspcd.c,v 1.87 2011/08/11 06:01:21 as-ewb Exp $

\file
Functions for reading DSP code from file asidsp.bin using HpiOs_ file functions

*/
/***********************************************************************/
#define SOURCEFILE_NAME "hpidspcd.c"
#include "hpidspcd.h"
#include "hpidebug.h"
#include "hpi_version.h"

#ifndef HPI_BUILD_KERNEL_MODE
#include <string.h>
#include <stdlib.h>
#endif

/* must match or be greater than the same constant in S2BIN.C */
#define BLOCK_LIMIT_DWORD 2048L


#ifndef DISABLE_PRAGMA_PACK1
#pragma pack(push, 1)
#endif

#ifndef DISABLE_PRAGMA_PACK1
#pragma pack(pop)
#endif

/** state specific to this loader */
struct dsp_code_private {
	/// File descriptor for dsp code file
	HpiOs_FILE file;
	/// dsp code buffer
	uint32_t *buffer;
};

/********************************************************************/
/* Load dspcode from file asidsp.bin */


/*-------------------------------------------------------------------*/

short HpiDspCode_Open(uint32_t nAdapter, void *pOsData,
		      struct dsp_code *dsp_code, uint32_t *pdwOsErrorCode)
{
	hpi_err_t err;
	int i, nRead = 0;
	struct code_header header;
	const char *pszFilepath;
	HpiOs_FILE file;

	pszFilepath = HpiOs_GetDspCodePath(nAdapter);
	HPI_DEBUG_LOG1(DEBUG, "Loading DSP code file: \"%s\"\n",pszFilepath);

	err = HpiOs_fopen_rb(pszFilepath, &file, pdwOsErrorCode);
	if (err)
		return (err);

	memset(&header, 0, sizeof(header));
	// seek for first CODE header
	for (i = 0; i < 1000; i++) {
		HpiOs_fseek(file, i, SEEK_SET);
		nRead = (int)HpiOs_fread(&header, 1, sizeof(header),
				file);
		if (nRead < sizeof(header)) {
			HPI_DEBUG_LOG0(DEBUG, "Ran out of data\n");
			break;
		}

		if (header.type == 0x45444F43)
			break;
	}
	if ((nRead < sizeof(header)) || (i == 1000)) {
		HpiOs_fclose(file);
		HPI_DEBUG_LOG0(DEBUG, "CODE header not found\n");
		return HPI_ERROR_DSP_FILE_NO_HEADER;
	}
	// walk CODE headers searching for correct one
	while ((nRead == sizeof(header)) && (header.adapter != nAdapter)) {
		HPI_DEBUG_LOG1(VERBOSE, "Skipping %04x ",
			       header.adapter);
		if (header.size == 0) {
			HpiOs_fclose(file);
			HPI_DEBUG_LOG0(DEBUG,
				       "NULL header found\n");
			return HPI_ERROR_DSP_FILE_NULL_HEADER;
		}
		HpiOs_fseek(file,
			    (header.size - sizeof(header)), SEEK_CUR);
		nRead =(int)HpiOs_fread(&header, 1, sizeof(header),
				file);
	}
	if ((nRead < sizeof(header)) || (header.adapter != nAdapter)) {
		HpiOs_fclose(file);
		HPI_DEBUG_LOG1(ERROR,
			       "Adapter type %04x not found\n",
			       nAdapter);
		return HPI_ERROR_DSP_SECTION_NOT_FOUND;
	}
	else
	{
		HPI_DEBUG_LOG1(VERBOSE,
			       "Adapter type %04x code found\n",
			       nAdapter);
	}

	dsp_code->pvt = HpiOs_MemAlloc(sizeof(*dsp_code->pvt));
	if (!dsp_code->pvt)
		return HPI_ERROR_MEMORY_ALLOC;

	dsp_code->pvt->buffer =
	    HpiOs_MemAlloc(sizeof(uint32_t) * BLOCK_LIMIT_DWORD);
	if (!dsp_code->pvt->buffer) {
		HpiOs_fclose(dsp_code->pvt->file);
		HPI_DEBUG_LOG0(ERROR, "Out of memory error\n");
		HpiOs_MemFree(dsp_code->pvt);
		return HPI_ERROR_MEMORY_ALLOC;
	}
	dsp_code->pvt->file = file;
	dsp_code->header = header;
	dsp_code->block_length = (header.size - sizeof(header)) / 4;
	dsp_code->word_count = 0;
	return 0;
}

/*-------------------------------------------------------------------*/
void HpiDspCode_Rewind(struct dsp_code *dsp_code)
{
	/* why not rewind and re-seek? */
	HpiDspCode_Close(dsp_code);
	HpiDspCode_Open(dsp_code->header.adapter, NULL, dsp_code, NULL);
}

/*-------------------------------------------------------------------*/
void HpiDspCode_Close(struct dsp_code *dsp_code)
{
	HpiOs_MemFree(dsp_code->pvt->buffer);
	HpiOs_fclose(dsp_code->pvt->file);
	HpiOs_MemFree(dsp_code->pvt);
}

/*-------------------------------------------------------------------*/
short HpiDspCode_ReadWord(struct dsp_code *dsp_code, uint32_t *pdwWord)
{
	size_t count;
	count = HpiOs_fread(pdwWord, sizeof(uint32_t), 1, dsp_code->pvt->file);
	if (count != 1)
		return (HPI_ERROR_DSP_FILE_FORMAT + 21);	// 231
	// could also calculate checksum here?
	dsp_code->word_count++;
	if (dsp_code->word_count > dsp_code->block_length)
		return (HPI_ERROR_DSP_FILE_FORMAT + 22);	// 232
	return 0;
}

/*-------------------------------------------------------------------*/
short HpiDspCode_ReadBlock(size_t nWordsRequested,
			   struct dsp_code *dsp_code, uint32_t **ppdwBlock)
{
	size_t count;

	if (nWordsRequested > BLOCK_LIMIT_DWORD)
		return (HPI_ERROR_DSP_FILE_FORMAT + 23);	// 233

	count = HpiOs_fread(dsp_code->pvt->buffer, sizeof(uint32_t), nWordsRequested,
			dsp_code->pvt->file);
	if (count != nWordsRequested)
		return (HPI_ERROR_DSP_FILE_FORMAT + 24);	// 234

	dsp_code->word_count += (long)nWordsRequested;
	if (dsp_code->word_count > dsp_code->block_length)
		return (HPI_ERROR_DSP_FILE_FORMAT + 25);	// 235

	*ppdwBlock = dsp_code->pvt->buffer;
	return (0);
}

// $Log: hpidspcd.c,v $
// Revision 1.87  2011/08/11 06:01:21  as-ewb
// Use hpi_version.h
//
// Revision 1.86  2011/05/02 21:34:15  as-dxb
// Fix log string's format size for adapter model
//
// Revision 1.85  2011/04/12 11:22:14  as-ewb
// Clean hpidspcode api
//
// Revision 1.84  2011/04/06 09:26:24  as-dxb
// Remove darwin specific DSP bootload code from generic HPI
//
// Revision 1.83  2011/04/06 04:50:51  as-ewb
// Move towards os independent dsp code loader API
//
// Revision 1.82  2011/04/06 03:36:56  as-ewb
// Moved linux kernel specific code into linux/drv/hpidspcd.c
//
// Revision 1.81  2011/04/05 23:01:22  as-ewb
// linux - relax firmware version check so current beta driver
// works with previous stable firmware
//
// Revision 1.80  2011/02/17 17:54:56  as-dxb
// Update HPI shared code with mac driver relevant changes
//
// Revision 1.79  2011/02/07 21:16:42  as-ewb
// Linux fail firmware load if image major.minor differs from driver.
// Formerly only major version mismatch was an error.
//
// Revision 1.78  2010/11/23 02:36:16  as-ewb
// Convert most conditional compilation define names to HPI_BUILD_x
//
// Revision 1.77  2010/11/17 00:20:27  as-ewb
// Switch to using C99 fixed width types, and some lowercase
// hpi type names.  (Previous defs are still defined for compatibility)
//
// Revision 1.76  2010/09/07 03:03:43  as-ewb
// switch to dev_printk for errors and warnings in linux firmware loader.
//
// Revision 1.75  2010/07/22 04:18:16  as-ewb
// Log firmware name etc only at debug level.
//
// Revision 1.74  2010/07/01 22:52:43  as-ewb
// Drop support for linux kernel version < 2.5.0
//
// Revision 1.73  2010/05/17 23:55:26  as-ewb
// Don't need null check for free(). (noted by smatch)
//
// Revision 1.72  2010/04/16 03:05:30  as-ewb
// Minor tweaks to ease (ALSA) style transform. Issue#1158
//
// Revision 1.71  2010/02/04 00:59:01  as-ewb
// Linux - error if firmware major version mismatch. Issue#620
//
// Revision 1.70  2008/07/15 01:15:30  as-ewb
// merge from branc30711 - standalone hpi.h, hpi header includes use double quotes, remove win98
//
// Revision 1.69.2.1  2008/07/04 05:31:16  as-ewb
// switch to hpi_internal. Remove win16, dspcode array loader. hpi.h now standalone
//
// Revision 1.69  2008/06/03 00:02:44  as-ewb
// Log warning if DSP bin version != driver version
//
// Revision 1.68  2008/03/28 12:56:34  as-age
// add debug output
//
// Revision 1.67  2008/03/12 23:35:04  as-ewb
// Remove DBG_TEXT on formats, now handled inside LOG macros
//
// Revision 1.66  2008/02/05 22:07:02  as-age
// remove HPI_4000
//
// Revision 1.65  2007/11/27 21:06:22  as-sgt
// SGT -get rid of warnings
//
// Revision 1.64  2007/11/27 09:16:25  as-ewb
// Replace LoadNNNN enum with existing HPI defines
//
// Revision 1.63  2007/11/26 20:14:25  as-ewb
// use struct dsp_code
//
// Revision 1.62  2007/11/23 04:26:34  as-ewb
// Ran Hpindent over all HPI files.
//
// Revision 1.61  2007/11/14 23:13:49  as-ewb
// reformat linux part.
//
// Revision 1.60  2007/10/25 21:01:12  as-age
// fix compiler errors for putting this code in a DLL.
//
// Revision 1.59  2007/10/11 20:08:21  as-sgt
// SGT - fixes for WIN16
//
// Revision 1.58  2007/10/08 20:03:34  as-ewb
// correct extern defs for 6400,8700 hex arrays
//
// Revision 1.57  2007/10/04 19:16:40  as-ewb
// Use 6400/8700 dsp code IDs
//
// Revision 1.56  2007/10/04 03:41:27  as-ewb
// Prep for transition from 64/8713  to -00 suffix names
//
// Revision 1.55  2007/09/07 09:43:25  as-ewb
// const correct dsp code path
//
// Revision 1.54  2007/06/01 17:38:52  as-age
// Remove HPI_ZLIB references.
//
// Revision 1.53  2007/05/18 16:53:31  as-sgt
// SGT - correct ax6200 & ax8800 hex file names
//
// Revision 1.52  2007/05/09 22:49:58  as-ewb
// tweaks to allow debug to print file name without path
//
// Revision 1.51  2007/04/12 22:49:00  as-ewb
// tweaks for asi2416 fw updater
//
// Revision 1.50  2007/03/30 18:54:42  as-sgt
// SGT - change DSPCODE_xxxx defines to HPI_DSPCODE_xxxx.  NOTE all HPI defines must start with HPI_.  Also, are these defines documented in Doxygen anyware?
//
// Revision 1.49  2007/03/26 13:48:52  as-tfe
// Initialize int variable with 0 not NULL in HpiDspCode_Open().  Fixes 64bit compile.
//
// Revision 1.48  2007/02/08 14:04:36  as-age
// 8900 code loading tweaks.
//
// Revision 1.47  2007/01/16 23:13:09  as-ewb
// linux always uses firmware loader
//
// Revision 1.46  2007/01/16 19:10:30  as-tfe
// Fix multiple adapter initialization under Vista.
//
// Revision 1.45  2006/11/24 04:57:17  as-ewb
// make usable for linux usermode
//
// Revision 1.44  2006/10/23 18:03:58  as-age
// Add ASI8800 back in.
//
// Revision 1.43  2006/10/17 21:02:16  as-ewb
// tweak debug log message
//
// Revision 1.42  2006/08/09 20:02:45  as-sgt
// SGT - add ASI6500 support (comments only - no code change).  Use HPI_INCLUDE_6600 for ASI6500
//
// Revision 1.41  2006/06/08 22:53:17  as-ewb
// remove support for 44,45,46,86,8800
//
// Revision 1.40  2006/06/02 19:08:06  as-tfe
// Check DSP section headers for zero size to avoid endless loop
//
// Revision 1.39  2006/03/09 14:15:35  as-tfe
// Added 'HW32 *pdwOsErrorCode' parameter to CreateAdapterObj(), HpiXXXX_BootHPI_ADAPTER_FAMILY_ASIDsp(),
// HpiDspCode_Open() and HpiOs_fopen_rb().
//
// Revision 1.38  2006/03/03 14:30:04  as-tfe
// Merged in ERROR_206_REDO branch.
//
// Revision 1.37.2.1  2006/03/02 20:53:21  as-tfe
// Split 206 error and modified HpiOs_fopen_rb() to return an error code.
//
// Revision 1.37  2006/02/21 00:23:43  as-ewb
// stringify now in hpi.h
//
// Revision 1.36  2006/02/10 08:06:26  as-ewb
// add asihpi/ prefix to filename for hotplug firmware loading
//
// Revision 1.35  2006/01/25 15:21:10  as-tfe
// Merged.
//
// Revision 1.31.4.10  2006/01/20 15:12:50  as-age
// Header dwCRC tweaks to sync with drv297rc10.
//
// Revision 1.31.4.9  2006/01/19 20:28:39  as-sgt
// sgt - add ax6600 support
//
// Revision 1.31.4.8  2006/01/19 19:57:35  as-sgt
// sgt - add AX6600 support for WIN16
//
// Revision 1.31.4.7  2005/12/14 04:22:16  as-ewb
// Use DISABLE_PRAGMA_PACK1 instead of include files.
// Only disabled for DSP code now.
//
// Revision 1.31.4.6  2005/12/12 15:52:46  as-tfe
// Moved pragma pack directives to include files hpipack1.h and hpipack_.h.
//
// Revision 1.31.4.5  2005/12/12 14:59:14  as-tfe
// Moved pragma pack directives to just around affected structure definitions.
//
// Revision 1.31.4.4  2005/11/29 23:55:26  as-ewb
// EWB restore code header walk for windows build + asidsp.bin
//
// Revision 1.31.4.3  2005/11/25 00:10:41  as-ewb
// EWB New debug macros, testing.
//
// Revision 1.31.4.2  2005/11/24 22:20:48  as-ewb
// EWB Fix some format strings for HW32
//
// Revision 1.31.4.1  2005/10/26 03:35:46  as-ewb
// EWB get rid of some HPI_ OS_LINUX conditional compilation
//
// Revision 1.31  2005/08/02 05:21:24  as-ewb
// ewb store code version from file in dspcode struct
//
// Revision 1.30  2005/07/07 20:01:48  as-age
// AGE - remove nDspCode_hex8705ArrayCount as well in HPI_INCLUDE_8700.
//
// Revision 1.29  2005/07/07 19:29:14  as-sgt
// SGT - change from HEX8705 to HEX6205 (for WHPI16, TASI etc)
//
// Revision 1.28  2005/06/15 22:20:57  as-ewb
// EWB remove paths with backslash from header  - confused doxygen
//
// Revision 1.27  2005/02/15 20:08:51  as-tfe
// TFE - Commented out call to printk().
//
// Revision 1.26  2005/02/09 00:22:51  as-ewb
// EWB remove compile warning
//
// Revision 1.25  2005/01/19 02:04:38  as-ewb
// EWB fix debug messages
//
// Revision 1.24  2004/12/16 19:04:19  as-age
// AGE - ax6205 hex file load correction.
//
// Revision 1.23  2004/12/13 16:01:47  as-age
// AGE - add 6205 code.
//
// Revision 1.22  2004/10/01 04:29:40  as-sgt
// SGT - add HFAR at line #325
//
// Revision 1.21  2004/09/14 00:05:41  as-ewb
// EWB correct externs for 6413 arrays
//
// Revision 1.20  2004/09/13 23:16:58  as-ewb
// EWB add hex6413 arrays for asi6400. Need to define HPI_INCLUDE_6400 to use.
//
// Revision 1.19  2004/09/08 14:52:57  as-tfe
// TFE - Fixed compile error.
//
// Revision 1.18  2004/09/08 03:11:49  as-ewb
// EWB made firmware load from file use same files as firmware loader
//
// Revision 1.17  2004/08/31 04:51:14  as-ewb
// EWB print error code from request firmware
//
// Revision 1.16  2004/08/30 23:53:13  as-ewb
// EWB allow asidsp.bin firmware loading as an option on linux
//
// Revision 1.15  2004/08/05 03:13:51  as-ewb
// EWB avoid use of pci_name()
//
// Revision 1.14  2004/07/29 04:12:33  as-ewb
// EWB tweaks to keep firmware loading compatible with linux 2.4.x
//
// Revision 1.13  2004/07/29 01:14:30  as-ewb
// EWB firmware loading version for linux
//
// Revision 1.12  2003/09/23 20:01:13  as-age
// AGE- add code to support hex file loading of 5000 and 8700.
//
// Revision 1.11  2003/05/30 19:46:29  as-age
// AGE - header and log area formatting updates.
//
// Revision 1.10  2003/05/29 23:50:13  as-ewb
// EWB regularize headers and logs for GPL processing.
// Main header uses slash-star, log uses slash-slash. .h file #endif comes before log
//
// Revision 1.9  2003/05/05 19:36:01  as-age
// AGE - in HpiDspCode_Open() change switch(nAdapter) to switch((HW16)nAdapter) to get
// BC 4.5 Win16 code to work again.
//
// Revision 1.8  2003/05/01 02:23:30  as-ewb
// EWB add load8600 option for array loading.
//
// Revision 1.7  2002/05/10 15:40:38  as-tfe
// TFE - Added typecast to eliminate compiler warning.
//
// Revision 1.6  2002/04/12 18:13:16  as-age
// AGE - added support for 8600 (for the moment it conditionally replaces 8800 code)
//
// Revision 1.5  2002/01/29 14:58:25  as-age
// AGE - Eliot's tidy up of asidps.bin code loader plus his Linux code for same.
//
// Revision 1.4  2002/01/28 19:37:43  as-age
// AGE - this version loads asidsp.bin files from Win NT kernel mode. So far only tested on
// a single DSP code file, so seek() stuff has not been wrung out. The following
// envorinment variables are used...
// ---Win16
// windir (typicallly = c:\windows), asidsp.bin path is
// c:\windows\system\asidsp.bin
// ---Win NT
// SystemRoot (typically = c:\winnt), asidsp.bin path is
// c:\winnt\system32\asidsp.bin
//
// Revision 1.3  2002/01/25 22:11:59  as-age
// AGE - abstracted fopen(),fread(),fseek(),feof() and fclose(). NT kernel code written,
// but not tested yet.
//
// Revision 1.2  2002/01/24 18:32:15  as-age
// AGE - minor tweak to support Win16 compile with USE_ASIDSP and USE_ZLIB both undefined.
//
// Revision 1.1  2002/01/24 04:32:52  as-ewb
// EWB module encapsulating DSP code access for bootloading.
//
//
